/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#pragma once
#include "Serializer.h"

namespace SLK
{
/*
class Blob
{
public:
	Blob()
	{
		this->size = 0;
		this->data = 0;
	}
	Blob(const void *data, int size)
	{
		this->size = size;
		this->data = new unsigned char[size];
		memcpy(this->data,data,size);

	}

	unsigned char *data;
	int size;
};
*/
std::ostream& operator<< (std::ostream& o, const Serializer& serializer);

class DBValue
{
public:
	DBValue(int dataType):
      m_dataType(dataType)
	{}

	int getDataType(){return m_dataType;};

	virtual std::string getAsString() = 0;
private:
	int m_dataType;//SQLITE_INTEGER etc
};

template <class T>
class DBValueType:public DBValue
{
public:
	DBValueType(T value, int dataType):
		DBValue(dataType),
		m_value(value)
	{}

	T getValue(){return m_value;};

	std::string getAsString()
	{
		std::stringstream stream;
		stream << m_value;
		return stream.str();
	}

private:
	T m_value;
};



class DBRow
{
public:
	DBRow(sqlite3_stmt *statement);
	virtual ~DBRow(void);

	template <class T>
	void Get(std::string name, T &value);
	void Get(std::string name, std::string &value);
private:
	typedef std::map<std::string,DBValue*> ValueMap;
	ValueMap m_values;
};

template <class T>
void DBRow::Get(std::string name, T &value)
{
	ValueMap::iterator iter = m_values.find(name);
	if(iter == m_values.end())
	{
		assert(0 && "missing DB column");
		throw SLK::Exception("Column %s not in database",name.c_str());
	}
	DBValueType<T> *val = dynamic_cast<DBValueType<T> *>(iter->second);
	if(val)
	{
		value = val->getValue();
	}
	else
	{
		std::string typeName(typeid(T).name());
		throw SLK::Exception("Column %s not a(n) %s type",name.c_str(),typeName.c_str());
	}
}

}
