/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#pragma once

#include "WindowToggler.h"
#include "Scripting.h"
#include "Singleton.h"
#include <list>
#include <stdarg.h>

namespace SLK
{

class DebugConsole:public Singleton<DebugConsole>
{
public:
	DebugConsole(CEGUI::Window* parent);
	virtual ~DebugConsole(void);

	WindowToggler* getWindowToggler(){return m_windowToggler;};
	void addText(const CEGUI::String& text);

	static void LogError(const asSMessageInfo *msg, void *param)
	{
		std::stringstream strStream;
		strStream << msg->message;
		const char *type = "ERR ";
		if( msg->type == asMSGTYPE_WARNING )
			getSingleton().addText("***WARNING***");
		else if( msg->type == asMSGTYPE_INFORMATION )
			getSingleton().addText("***INFORMATION***");
		else
		{
			getSingleton().addText("***ERROR***");
			strStream<< " row: " << msg->row;
		}
		getSingleton().addText(strStream.str());
	}

	static void Print(const std::string &text)
	{
		getSingleton().addText(text);
	}

	static void Print(int num)
	{
		std::stringstream strStream;
		strStream << num;

		getSingleton().addText(strStream.str());
	}

	static void Print(unsigned int num)
	{
		std::stringstream strStream;
		strStream << num;

		getSingleton().addText(strStream.str());
	}

	static void Print(bool num)
	{
		if(num)
			getSingleton().addText("true");
		else
			getSingleton().addText("false");
	}

	static void Print(float num)
	{
		std::stringstream strStream;
		strStream << num;

		getSingleton().addText(strStream.str());
	}
	
	static void Print(double num)
	{
		std::stringstream strStream;
		strStream << num;

		getSingleton().addText(strStream.str());
	}

	static void Print()
	{
		getSingleton().addText("");
	}

	static void Print(const char *text, ...)
	{
		char formattedText[2024];     
		va_list varList;    

		// Expand the format string   
		va_start(varList, text);   
		vsprintf(formattedText, text, varList);   
		va_end(varList); 

		getSingleton().addText(formattedText);
	}

private:

	bool onTextAdded(const CEGUI::EventArgs& args);
	bool onShown(const CEGUI::EventArgs& args);
	void addHistory(std::string text);

	const static int MAX_HISTORY = 100;
	const static int MAX_INPUT_HISTORY = 5;

	WindowToggler *m_windowToggler;
	CEGUI::Window *m_screen;
	CEGUI::Listbox *m_listBox;
	CEGUI::Editbox *m_inputBox;

	std::string m_script;
	bool m_multilineScript;

	std::string m_inputHistory[MAX_INPUT_HISTORY];
	int m_historyIndex;
	int m_historySize;
};


}