/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#pragma once
#include <vector>
#include "PathMapTile.h"
#include "Platform.h"

namespace SLK
{
//Based on stl-based priority queue from Game Programming Gems page 283-284

class TileSorter
{
public:
	bool operator()(PathMapTile* first, PathMapTile* second)
	{
		return (first->total)>(second->total);
	}
};

class SerilynExport PriorityQueue
{
public:
	PriorityQueue(void);
	virtual ~PriorityQueue(void);

	inline PathMapTile *pop()
	{
		PathMapTile * tile= m_heap.front();
		std::pop_heap(m_heap.begin(), m_heap.end(),TileSorter());
		m_heap.pop_back();

		return tile;
	}

	inline void push(PathMapTile *tile)
	{
		m_heap.push_back(tile);
		std::push_heap(m_heap.begin(), m_heap.end(),TileSorter());
	}

	inline void update(PathMapTile *element)
	{
		std::vector<PathMapTile*>::iterator iter;
		for(iter = m_heap.begin(); iter!=m_heap.end(); iter++)
		{
			if((*iter)->x == element->x && (*iter)->y == element->y)
			{
				std::push_heap(m_heap.begin(), iter+1, TileSorter());
				return;
			}
		}
	}

	inline bool isEmpty()
	{
		return m_heap.empty();
	}

	//HACK - should be removed when rest of article is implemented
	inline std::vector<PathMapTile *> &getHeap(){return m_heap;};
private:
	std::vector<PathMapTile *> m_heap;
};


}
