/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#include "StdAfx.h"
#include "AStarPathfind.h"


namespace SLK
{
AStarPathfind* AStarPathfind::getSingletonPtr()
{
	return m_singleton;
}

AStarPathfind& AStarPathfind::getSingleton()
{
	assert(m_singleton); 
	return (*m_singleton);
}


AStarPathfind::AStarPathfind(Map2D* map):
	m_map(map),
	m_tilesCreated(0)
{
	m_size = map->getXLength();
}

AStarPathfind::~AStarPathfind(void)
{
	clear();
	for(unsigned int i=0; i<m_pool.size(); i++)
	{
		SAFEDELETE(m_pool[i]);
	}
}

void AStarPathfind::clear()
{
	while(!m_open.isEmpty())
	{
		PathMapTile* cur = m_open.pop();
	}
	while(m_master.size()>0)
	{
		stdext::hash_map<int, PathMapTile*>::iterator cur = m_master.begin();
		m_pool.push_back(cur->second);
		m_master.erase(cur);
	}
}



float AStarPathfind::getPath(int startX, int startY, int endX, int endY, std::vector<IntVector> &path)
{
	if(!m_map->canWalkBetween(startX,startY,endX,endY))
		return -1;

	clear();
	m_endX = endX;
	m_endY = endY;
	PathMapTile *startNode = getTile(startX, startY);
	startNode->onOpen=true;
	startNode->onClosed = true;
	startNode->parent = 0;
	startNode->cost = 0;
	startNode->total = getHeuristic(startX,startY);
	m_open.push(startNode);

	while(!m_open.isEmpty())
	{
		PathMapTile *tile = m_open.pop();

		if(tile->x == endX && tile->y == endY)
		{
			constructPath(tile, path);
			return tile->cost;
		}

		//loop through connecting nodes
		checkNode(tile->x+1,tile->y,tile);
		checkNode(tile->x-1,tile->y,tile);
		checkNode(tile->x,tile->y+1,tile);
		checkNode(tile->x,tile->y-1,tile);
		checkNode(tile->x-1,tile->y-1,tile,NorthWest);
		checkNode(tile->x+1,tile->y-1,tile,NorthEast);
		checkNode(tile->x-1,tile->y+1,tile,SouthWest);
		checkNode(tile->x+1,tile->y+1,tile,SouthEast);

		tile->onClosed = true;
	}

	return -1;
}


void AStarPathfind::constructPath(PathMapTile *endNode, std::vector<IntVector> &path)
{
	while(endNode)
	{
		path.push_back(IntVector(endNode->x, endNode->y));
		endNode = endNode->parent;
	}
}




}