/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#include "StdAfx.h"
#include "HotkeyManager.h"

namespace SLK
{

HotkeyManager::HotkeyManager(OIS::Keyboard *keyboard):
	m_modifiers(0),
	m_keyboard(keyboard)
{
}

HotkeyManager::~HotkeyManager(void)
{
}


void HotkeyManager::pressKey(OIS::KeyCode keycode)
{
	if(keycode == OIS::KC_LSHIFT || keycode == OIS::KC_RSHIFT)
	{
		m_modifiers |= KeyModifier::BitsShift;
		for(int i=0; i<NUM_KEY_ASSIGNMENTS; i++)
		{
			if(m_keyboard->isKeyDown((OIS::KeyCode)i) && m_hotkeys[KeyModifier::Shift][i].isConnected())
				m_hotkeys[KeyModifier::Shift][i].doAction(HotkeyEventArgs((OIS::KeyCode)i,HotkeyEventArgs::Down, KeyModifier::Shift));
		}
		if(m_hotkeys[KeyModifier::None][keycode].isConnected())
			m_hotkeys[KeyModifier::None][keycode].doAction(HotkeyEventArgs(keycode,HotkeyEventArgs::Down, KeyModifier::None));

	}
	else if(keycode == OIS::KC_LMENU || keycode == OIS::KC_RMENU)
	{
		m_modifiers |= KeyModifier::BitsAlt;
		for(int i=0; i<NUM_KEY_ASSIGNMENTS; i++)
		{
			if(m_keyboard->isKeyDown((OIS::KeyCode)i) && m_hotkeys[KeyModifier::Alt][i].isConnected())
				m_hotkeys[KeyModifier::Alt][i].doAction(HotkeyEventArgs((OIS::KeyCode)i,HotkeyEventArgs::Down, KeyModifier::Alt));
		}
		if(m_hotkeys[KeyModifier::None][keycode].isConnected())
			m_hotkeys[KeyModifier::None][keycode].doAction(HotkeyEventArgs(keycode,HotkeyEventArgs::Down, KeyModifier::None));

	}
	else if(keycode == OIS::KC_LCONTROL || keycode == OIS::KC_RCONTROL)
	{
		m_modifiers |= KeyModifier::BitsCtrl;
		for(int i=0; i<NUM_KEY_ASSIGNMENTS; i++)
		{
			if(m_keyboard->isKeyDown((OIS::KeyCode)i) && m_hotkeys[KeyModifier::Ctrl][i].isConnected())
				m_hotkeys[KeyModifier::Ctrl][i].doAction(HotkeyEventArgs((OIS::KeyCode)i,HotkeyEventArgs::Down, KeyModifier::Ctrl));
		}
		if(m_hotkeys[KeyModifier::None][keycode].isConnected())
			m_hotkeys[KeyModifier::None][keycode].doAction(HotkeyEventArgs(keycode,HotkeyEventArgs::Down, KeyModifier::None));

	}

	//it's not a control key so notify it it was pressed with any controls that are down
	if(m_modifiers & KeyModifier::BitsShift && m_hotkeys[KeyModifier::Shift][keycode].isConnected())
		m_hotkeys[KeyModifier::Shift][keycode].doAction(HotkeyEventArgs(keycode,HotkeyEventArgs::Down, KeyModifier::Shift));
	if(m_modifiers & KeyModifier::BitsAlt && m_hotkeys[KeyModifier::Alt][keycode].isConnected())
		m_hotkeys[KeyModifier::Alt][keycode].doAction(HotkeyEventArgs(keycode,HotkeyEventArgs::Down, KeyModifier::Alt));
	if(m_modifiers & KeyModifier::BitsCtrl && m_hotkeys[KeyModifier::Ctrl][keycode].isConnected())
		m_hotkeys[KeyModifier::Ctrl][keycode].doAction(HotkeyEventArgs(keycode,HotkeyEventArgs::Down, KeyModifier::Ctrl));
	if(!m_modifiers && m_hotkeys[KeyModifier::None][keycode].isConnected())
		m_hotkeys[KeyModifier::None][keycode].doAction(HotkeyEventArgs(keycode,HotkeyEventArgs::Down, KeyModifier::None));


}

void HotkeyManager::releaseKey(OIS::KeyCode keycode)
{
	if(keycode == OIS::KC_LSHIFT || keycode == OIS::KC_RSHIFT)
	{
		m_modifiers &= ~KeyModifier::BitsShift;
		for(int i=0; i<NUM_KEY_ASSIGNMENTS; i++)
		{
			if(m_keyboard->isKeyDown((OIS::KeyCode)i) && m_hotkeys[KeyModifier::Shift][i].isConnected())
				m_hotkeys[KeyModifier::Shift][i].doAction(HotkeyEventArgs((OIS::KeyCode)i,HotkeyEventArgs::Up, KeyModifier::Shift));
		}
	}
	else if(keycode == OIS::KC_LMENU || keycode == OIS::KC_RMENU)
	{
		m_modifiers &= ~KeyModifier::BitsAlt;
		for(int i=0; i<NUM_KEY_ASSIGNMENTS; i++)
		{
			if(m_keyboard->isKeyDown((OIS::KeyCode)i) && m_hotkeys[KeyModifier::Alt][i].isConnected())
				m_hotkeys[KeyModifier::Alt][i].doAction(HotkeyEventArgs((OIS::KeyCode)i,HotkeyEventArgs::Up, KeyModifier::Alt));
		}
	}
	else if(keycode == OIS::KC_LCONTROL || keycode == OIS::KC_RCONTROL)
	{
		m_modifiers &= ~KeyModifier::BitsCtrl;
		for(int i=0; i<NUM_KEY_ASSIGNMENTS; i++)
		{
			if(m_keyboard->isKeyDown((OIS::KeyCode)i) && m_hotkeys[KeyModifier::Ctrl][i].isConnected())
				m_hotkeys[KeyModifier::Ctrl][i].doAction(HotkeyEventArgs((OIS::KeyCode)i,HotkeyEventArgs::Up, KeyModifier::Ctrl));
		}
	}

	//it's not a control key so notify it it was pressed with any controls that are down
	if(m_modifiers & KeyModifier::BitsShift && m_hotkeys[KeyModifier::Shift][keycode].isConnected())
		m_hotkeys[KeyModifier::Shift][keycode].doAction(HotkeyEventArgs(keycode,HotkeyEventArgs::Up, KeyModifier::Shift));
	if(m_modifiers & KeyModifier::BitsAlt && m_hotkeys[KeyModifier::Alt][keycode].isConnected())
		m_hotkeys[KeyModifier::Alt][keycode].doAction(HotkeyEventArgs(keycode,HotkeyEventArgs::Up, KeyModifier::Alt));
	if(m_modifiers & KeyModifier::BitsCtrl && m_hotkeys[KeyModifier::Ctrl][keycode].isConnected())
		m_hotkeys[KeyModifier::Ctrl][keycode].doAction(HotkeyEventArgs(keycode,HotkeyEventArgs::Up, KeyModifier::Ctrl));
	if(!m_modifiers && m_hotkeys[KeyModifier::None][keycode].isConnected())
		m_hotkeys[KeyModifier::None][keycode].doAction(HotkeyEventArgs(keycode,HotkeyEventArgs::Up, KeyModifier::None));

}

void HotkeyManager::moveMouse(OIS::MouseState mouseState)
{
	//Do actions
	if(m_mouseEvent.isConnected())
		m_mouseEvent.doAction(MouseEventArgs(mouseState));
}

void HotkeyManager::pressMouse(OIS::MouseState mouseState, OIS::MouseButtonID mouseButton)
{
	if(m_modifiers & KeyModifier::BitsShift && m_mouseButtons[KeyModifier::Shift][mouseButton].isConnected())
		m_mouseButtons[KeyModifier::Shift][mouseButton].doAction(MouseEventArgs(mouseButton,MouseEventArgs::Down, KeyModifier::Shift, mouseState));
	if(m_modifiers & KeyModifier::BitsAlt && m_mouseButtons[KeyModifier::Alt][mouseButton].isConnected())
		m_mouseButtons[KeyModifier::Alt][mouseButton].doAction(MouseEventArgs(mouseButton,MouseEventArgs::Down, KeyModifier::Alt, mouseState));
	if(m_modifiers & KeyModifier::BitsCtrl && m_mouseButtons[KeyModifier::Ctrl][mouseButton].isConnected())
		m_mouseButtons[KeyModifier::Ctrl][mouseButton].doAction(MouseEventArgs(mouseButton,MouseEventArgs::Down, KeyModifier::Ctrl, mouseState));
	if(!m_modifiers && m_mouseButtons[KeyModifier::None][mouseButton].isConnected())
		m_mouseButtons[KeyModifier::None][mouseButton].doAction(MouseEventArgs(mouseButton,MouseEventArgs::Down, KeyModifier::None, mouseState));
}

void HotkeyManager::releaseMouse(OIS::MouseState mouseState, OIS::MouseButtonID mouseButton)
{
	if(m_modifiers & KeyModifier::BitsShift && m_mouseButtons[KeyModifier::Shift][mouseButton].isConnected())
		m_mouseButtons[KeyModifier::Shift][mouseButton].doAction(MouseEventArgs(mouseButton,MouseEventArgs::Up, KeyModifier::Shift, mouseState));
	if(m_modifiers & KeyModifier::BitsAlt && m_mouseButtons[KeyModifier::Alt][mouseButton].isConnected())
		m_mouseButtons[KeyModifier::Alt][mouseButton].doAction(MouseEventArgs(mouseButton,MouseEventArgs::Up, KeyModifier::Alt, mouseState));
	if(m_modifiers & KeyModifier::BitsCtrl && m_mouseButtons[KeyModifier::Ctrl][mouseButton].isConnected())
		m_mouseButtons[KeyModifier::Ctrl][mouseButton].doAction(MouseEventArgs(mouseButton,MouseEventArgs::Up, KeyModifier::Ctrl, mouseState));
	if(!m_modifiers && m_mouseButtons[KeyModifier::None][mouseButton].isConnected())
		m_mouseButtons[KeyModifier::None][mouseButton].doAction(MouseEventArgs(mouseButton,MouseEventArgs::Up, KeyModifier::None, mouseState));
}


bool HotkeyManager::checkHotkey(OIS::KeyCode key, KeyModifier::Modifier modifier)
{
	assert(key<NUM_KEY_ASSIGNMENTS);
	return m_hotkeys[modifier][key].isConnected();
}



bool HotkeyManager::checkMouseButton(OIS::MouseButtonID button, KeyModifier::Modifier modifier)
{
	return m_mouseButtons[modifier][button].isConnected();
}


bool HotkeyManager::checkMouseMove()
{
	return m_mouseEvent.isConnected();
}

void HotkeyManager::unassignHotkey(OIS::KeyCode key, KeyModifier::Modifier modifier)
{
	assert(key<NUM_KEY_ASSIGNMENTS);
	m_hotkeys[modifier][key].disconnect();
}


void HotkeyManager::unassignMouseButton(OIS::MouseButtonID button, KeyModifier::Modifier modifier)
{
	m_mouseButtons[modifier][button].disconnect();
}

void HotkeyManager::unassignMouseMove()
{
	m_mouseEvent.disconnect();
}

}