/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#include "StdAfx.h"

#include "Angelscript.h"
#include "OgreScriptSupport.h"
#include "Ogre.h"
#include "Scripting.h"

namespace SLK
{

#undef GetObject//Undefine the windows macro

OgreScriptSupport::OgreScriptSupport(void)
{
}

OgreScriptSupport::~OgreScriptSupport(void)
{
}

static void Vector3DefaultConstructor(Ogre::Vector3 *self)
{
	new(self) Ogre::Vector3();
}

static void Vector3CopyConstructor(const Ogre::Vector3 &other, Ogre::Vector3 *self)
{
	new(self) Ogre::Vector3(other);
}

static void Vector3InitConstructor(float x, float y, float z, Ogre::Vector3 *self)
{
	new(self) Ogre::Vector3(x,y,z);
}
/*
static void Vector3DefaultConstructor_Generic(asIScriptGeneric *gen)
{
	Ogre::Vector3 *self = (Ogre::Vector3*)gen->GetObject();
	new(self) Ogre::Vector3();
}

static void Vector3CopyConstructor_Generic(asIScriptGeneric *gen)
{
	Ogre::Vector3 *other = (Ogre::Vector3*)gen->GetArgObject(0);
	Ogre::Vector3 *self = (Ogre::Vector3*)gen->GetObject();
	Vector3CopyConstructor(*other, self);
}

static void Vector3InitConstructor_Generic(asIScriptGeneric *gen)
{
	float x = gen->GetArgFloat(0);
	float y = gen->GetArgFloat(1);
	float z = gen->GetArgFloat(2);
	Ogre::Vector3 *self = (Ogre::Vector3*)gen->GetObject();
	Vector3InitConstructor(x,y,z,self);
}*/

static bool equals(const Ogre::Vector3 &a, const Ogre::Vector3 &b)
{
	return a==b;
}

static bool notEquals(const Ogre::Vector3 &a, const Ogre::Vector3 &b)
{
	return a!=b;
}

static Ogre::Vector3 add(const Ogre::Vector3 &a, const Ogre::Vector3 &b)
{
	return a+b;
}

static Ogre::Vector3 subtract(const Ogre::Vector3 &a, const Ogre::Vector3 &b)
{
	return a-b;
}

static Ogre::Vector3 multiply(const Ogre::Vector3 &a, float b)
{
	return a*b;
}

static Ogre::Vector3 multiply(float a, const Ogre::Vector3 &b)
{
	return a*b;
}

static Ogre::Vector3 divide(const Ogre::Vector3 &a, float b)
{
	return a/b;
}

void OgreScriptSupport::RegisterVector3()
{
	// Register the type
	Scripting::getSingleton().RegisterObjectType("vector3", sizeof(Ogre::Vector3), asOBJ_VALUE | asOBJ_POD | asOBJ_APP_CLASS_CA);

	// Register the object properties
	Scripting::getSingleton().RegisterObjectProperty("vector3", "float x", offsetof(Ogre::Vector3, x));
	Scripting::getSingleton().RegisterObjectProperty("vector3", "float y", offsetof(Ogre::Vector3, y));
	Scripting::getSingleton().RegisterObjectProperty("vector3", "float z", offsetof(Ogre::Vector3, z));

	// Register the object operator overloads
	Scripting::getSingleton().RegisterObjectBehaviour("vector3", asBEHAVE_CONSTRUCT,  "void f()",                     asFUNCTION(Vector3DefaultConstructor), asCALL_CDECL_OBJLAST);
	Scripting::getSingleton().RegisterObjectBehaviour("vector3", asBEHAVE_CONSTRUCT,  "void f(const vector3 &in)",       asFUNCTION(Vector3CopyConstructor), asCALL_CDECL_OBJLAST);
	Scripting::getSingleton().RegisterObjectBehaviour("vector3", asBEHAVE_CONSTRUCT,  "void f(float, float, float)",  asFUNCTION(Vector3InitConstructor), asCALL_CDECL_OBJLAST);

	// Register the operator overloads
	Scripting::getSingleton().RegisterObjectMethod("vector3", "vector3 &opAddAssign(const vector3 &in)",      asMETHODPR(Ogre::Vector3,operator+=,(const Ogre::Vector3 &),Ogre::Vector3&), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("vector3", "vector3 &opSubAssign(const vector3 &in)",      asMETHODPR(Ogre::Vector3,operator-=,(const Ogre::Vector3 &),Ogre::Vector3&), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("vector3", "vector3 &opMulAssign(float)",               asMETHODPR(Ogre::Vector3,operator*=,(float),Ogre::Vector3&), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("vector3", "vector3 &opDivAssign(float)",               asMETHODPR(Ogre::Vector3,operator/=,(float),Ogre::Vector3&), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("vector3", "bool opEquals(const vector3 &in) const", asFUNCTIONPR(equals, (const Ogre::Vector3&, const Ogre::Vector3&), bool), asCALL_CDECL_OBJFIRST);
	Scripting::getSingleton().RegisterObjectMethod("vector3", "vector3 opAdd(const vector3 &in) const", asFUNCTIONPR(add, (const Ogre::Vector3&, const Ogre::Vector3&), Ogre::Vector3), asCALL_CDECL_OBJFIRST);
	Scripting::getSingleton().RegisterObjectMethod("vector3", "vector3 opSub(const vector3 &in) const", asFUNCTIONPR(subtract, (const Ogre::Vector3&, const Ogre::Vector3&), Ogre::Vector3), asCALL_CDECL_OBJFIRST);
	Scripting::getSingleton().RegisterObjectMethod("vector3", "vector3 opMul(float) const",          asFUNCTIONPR(multiply, (const Ogre::Vector3 &, float), Ogre::Vector3), asCALL_CDECL_OBJFIRST);
	Scripting::getSingleton().RegisterObjectMethod("vector3", "vector3 opMul_r(float) const",          asFUNCTIONPR(multiply, (float, const Ogre::Vector3 &), Ogre::Vector3), asCALL_CDECL_OBJLAST);
	Scripting::getSingleton().RegisterObjectMethod("vector3", "vector3 opDiv(float) const",          asFUNCTIONPR(divide, (const Ogre::Vector3 &, float), Ogre::Vector3), asCALL_CDECL_OBJFIRST);

	// Register the object methods
	Scripting::getSingleton().RegisterObjectMethod("vector3", "float length() const", asMETHOD(Ogre::Vector3,length), asCALL_THISCALL);
}

static void Vector2DefaultConstructor(Ogre::Vector2 *self)
{
	new(self) Ogre::Vector2();
}

static void Vector2CopyConstructor(const Ogre::Vector2 &other, Ogre::Vector2 *self)
{
	new(self) Ogre::Vector2(other);
}

static void Vector2InitConstructor(float x, float y, Ogre::Vector2 *self)
{
	new(self) Ogre::Vector2(x,y);
}
/*
static void Vector2DefaultConstructor_Generic(asIScriptGeneric *gen)
{
	Ogre::Vector2 *self = (Ogre::Vector2*)gen->GetObject();
	new(self) Ogre::Vector2();
}

static void Vector2CopyConstructor_Generic(asIScriptGeneric *gen)
{
	Ogre::Vector2 *other = (Ogre::Vector2*)gen->GetArgObject(0);
	Ogre::Vector2 *self = (Ogre::Vector2*)gen->GetObject();
	Vector2CopyConstructor(*other, self);
}

static void Vector2InitConstructor_Generic(asIScriptGeneric *gen)
{
	float x = gen->GetArgFloat(0);
	float y = gen->GetArgFloat(1);
	Ogre::Vector2 *self = (Ogre::Vector2*)gen->GetObject();
	Vector2InitConstructor(x,y,self);
}
*/
static bool equals(const Ogre::Vector2 &a, const Ogre::Vector2 &b)
{
	return a==b;
}

static bool notEquals(const Ogre::Vector2 &a, const Ogre::Vector2 &b)
{
	return a!=b;
}

static Ogre::Vector2 add(const Ogre::Vector2 &a, const Ogre::Vector2 &b)
{
	return a+b;
}

static Ogre::Vector2 subtract(const Ogre::Vector2 &a, const Ogre::Vector2 &b)
{
	return a-b;
}

static Ogre::Vector2 multiply(const Ogre::Vector2 &a, float b)
{
	return a*b;
}

static Ogre::Vector2 multiply(float a, const Ogre::Vector2 &b)
{
	return a*b;
}

static Ogre::Vector2 divide(const Ogre::Vector2 &a, float b)
{
	return a/b;
}

void OgreScriptSupport::RegisterVector2()
{
	// Register the type
	Scripting::getSingleton().RegisterObjectType("vector2", sizeof(Ogre::Vector2), asOBJ_VALUE | asOBJ_POD | asOBJ_APP_CLASS_CA);

	// Register the object properties
	Scripting::getSingleton().RegisterObjectProperty("vector2", "float x", offsetof(Ogre::Vector2, x));
	Scripting::getSingleton().RegisterObjectProperty("vector2", "float y", offsetof(Ogre::Vector2, y));

	// Register the object operator overloads
	Scripting::getSingleton().RegisterObjectBehaviour("vector2", asBEHAVE_CONSTRUCT,  "void f()",                     asFUNCTION(Vector2DefaultConstructor), asCALL_CDECL_OBJLAST);
	Scripting::getSingleton().RegisterObjectBehaviour("vector2", asBEHAVE_CONSTRUCT,  "void f(const vector2 &in)",       asFUNCTION(Vector2CopyConstructor), asCALL_CDECL_OBJLAST);
	Scripting::getSingleton().RegisterObjectBehaviour("vector2", asBEHAVE_CONSTRUCT,  "void f(float, float, float)",  asFUNCTION(Vector2InitConstructor), asCALL_CDECL_OBJLAST);

	// Register the operator overloads
	Scripting::getSingleton().RegisterObjectMethod("vector2", "vector2 &opAddAssign(const vector2 &in)",      asMETHODPR(Ogre::Vector2,operator+=,(const Ogre::Vector2 &),Ogre::Vector2&), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("vector2", "vector2 &opSubAssign(const vector2 &in)",      asMETHODPR(Ogre::Vector2,operator-=,(const Ogre::Vector2 &),Ogre::Vector2&), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("vector2", "vector2 &opMulAssign(float)",               asMETHODPR(Ogre::Vector2,operator*=,(float),Ogre::Vector2&), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("vector2", "vector2 &opDivAssign(float)",               asMETHODPR(Ogre::Vector2,operator/=,(float),Ogre::Vector2&), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("vector2", "bool opEquals(const vector2 &in) const", asFUNCTIONPR(equals, (const Ogre::Vector2&, const Ogre::Vector2&), bool), asCALL_CDECL_OBJFIRST);
	Scripting::getSingleton().RegisterObjectMethod("vector2", "vector2 opAdd(const vector2 &in) const", asFUNCTIONPR(add, (const Ogre::Vector2&, const Ogre::Vector2&), Ogre::Vector2), asCALL_CDECL_OBJFIRST);
	Scripting::getSingleton().RegisterObjectMethod("vector2", "vector2 opSub(const vector2 &in) const", asFUNCTIONPR(subtract, (const Ogre::Vector2&, const Ogre::Vector2&), Ogre::Vector2), asCALL_CDECL_OBJFIRST);
	Scripting::getSingleton().RegisterObjectMethod("vector2", "vector2 opMul(float) const",          asFUNCTIONPR(multiply, (const Ogre::Vector2 &, float), Ogre::Vector2), asCALL_CDECL_OBJFIRST);
	Scripting::getSingleton().RegisterObjectMethod("vector2", "vector2 opMul_r(float) const",          asFUNCTIONPR(multiply, (float, const Ogre::Vector2 &), Ogre::Vector2), asCALL_CDECL_OBJLAST);
	Scripting::getSingleton().RegisterObjectMethod("vector2", "vector2 opDiv(float) const",          asFUNCTIONPR(divide, (const Ogre::Vector2 &, float), Ogre::Vector2), asCALL_CDECL_OBJFIRST);

	// Register the object methods
	Scripting::getSingleton().RegisterObjectMethod("vector2", "float length() const", asMETHOD(Ogre::Vector2,length), asCALL_THISCALL);
}

}
