/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#include "StdAfx.h"
#include "ScreenWindow.h"
#include "Application.h"

namespace SLK
{

ScreenWindow::ScreenWindow(CEGUI::Window *window, Ogre::SceneNode *node, bool useBoundingBox, int farDistance):
	m_window(window),
	m_node(node),
	m_useBoundingBox(useBoundingBox),
	m_enabled(true),
	m_farDistanceSquared(farDistance * farDistance)
{
}

ScreenWindow::~ScreenWindow(void)
{
}

void ScreenWindow::update()
{
	if (!m_enabled)
		return;

	Ogre::Camera* camera = Application::getSingletonPtr()->getCamera();
	
	Ogre::Vector3 pos = m_node->getPosition();

	bool tempHide = false;
	if(camera->getDerivedPosition().squaredDistance(pos) > m_farDistanceSquared)
	{
		tempHide = true;
	}
	else
	{
		// Is the camera facing that point? If not, hide the overlay and return.
		Ogre::Plane cameraPlane = Ogre::Plane(Ogre::Vector3(camera->getDerivedOrientation().zAxis()), camera->getDerivedPosition());
		if(cameraPlane.getSide(pos) != Ogre::Plane::NEGATIVE_SIDE)
		{
			tempHide = true;
		}
	}

	if(tempHide)
	{
		m_window->hide();
		return;
	}
	else
	{
		m_window->show();
	}

	if(m_useBoundingBox)
	{
		// get the projection of the object's AABB into screen space
		const Ogre::AxisAlignedBox& bbox = m_node->_getWorldAABB();
		if(!bbox.isNull())
			pos.y = bbox.getMaximum().y;

		pos.y += bbox.getSize().y;
	}

	Ogre::Vector4 transPos = camera->getProjectionMatrix() * camera->getViewMatrix() * Ogre::Vector4(pos);
	transPos = transPos/transPos.w;

	Ogre::Vector2 topLeft = Ogre::Vector2((transPos.x+1)*0.5, 1-(transPos.y+1)/2);

	m_window->setPosition(CEGUI::UVector2(CEGUI::UDim(topLeft.x-0.05/2,0),CEGUI::UDim(topLeft.y,0)));

}

}